/**
  @file qapi_device_info.h

 EXTERNALIZED FUNCTIONS
  none

 INITIALIZATION AND SEQUENCING REQUIREMENTS
  none
*/

/*
 Copyright (c) 2017  Qualcomm Technologies, Inc.  All Rights Reserved.
 Confidential and Proprietary - Qualcomm Technologies, Inc.
 ======================================================================*/
/*======================================================================
 *
 *        EDIT HISTORY FOR FILE
 *
 *	 This section contains comments describing changes made to the
 *	 module. Notice that changes are listed in reverse chronological
 *	 order.
 *
 *	
 *
 *
 * when 		who 	what, where, why
 * ---------- 	---   ------------------------------------------------
 * 2017-04-25   leo   (Tech Comm) Edited/added Doxygen comments and markup.
 ======================================================================*/
 
#ifndef _QAPI_DEVICE_INFO_H
#define _QAPI_DEVICE_INFO_H

#include "stdbool.h"
#include "qapi/qapi_data_txm_base.h"
#include "qapi/qapi_status.h"

#define  TXM_QAPI_DEVICE_INFO_INIT               TXM_QAPI_DEVICE_INFO_BASE + 1
#define  TXM_QAPI_DEVICE_INFO_GET                TXM_QAPI_DEVICE_INFO_BASE + 2
#define  TXM_QAPI_DEVICE_INFO_RELEASE            TXM_QAPI_DEVICE_INFO_BASE + 3
#define  TXM_QAPI_DEVICE_INFO_SET_CB             TXM_QAPI_DEVICE_INFO_BASE + 4
#define  TXM_QAPI_DEVICE_INFO_RESET              TXM_QAPI_DEVICE_INFO_BASE + 5

/** Valid values for network service status (QAPI_DEVICE_INFO_SERVICE_STATE_E). */
typedef enum
{
  QAPI_DEVICE_INFO_SRV_STATE_NO_SRV = 0, /**< No service. */
  QAPI_DEVICE_INFO_SRV_STATE_SRV = 1     /**< Service is available. */
}srv_status;


/** Valid values for network indication (QAPI_DEVICE_INFO_NETWORK_IND_E). */
typedef enum
{
  QAPI_DEVICE_INFO_NW_IND_NO_SRV = 0, /**< No service. */
  QAPI_DEVICE_INFO_NW_IND_SRV = 1     /**< Service is available. */
}nw_indication;


/** Valid values for network RRC state. */
typedef enum
{
  QAPI_DEVICE_INFO_RRC_IDLE = 0,       /**< Status: Idle. */
  QAPI_DEVICE_INFO_RRC_CONNECTED = 1   /**< Status: connected. */
}rrc_state;


/** Valid values for network Extended Mobility Management (EMM) state. */
typedef enum
{
  QAPI_DEVICE_INFO_EMM_NULL = 0,                              /**< Null. */
  QAPI_DEVICE_INFO_EMM_DEREGISTERED = 1,                      /**< Deregistered. */
  QAPI_DEVICE_INFO_EMM_REGISTERED_INITIATED = 2,              /**< Registered, initiated. */
  QAPI_DEVICE_INFO_EMM_REGISTERED = 3,                        /**< Registered. */
  QAPI_DEVICE_INFO_EMM_TRACKING_AREA_UPDATING_INITIATED = 4,  /**< Tracking area update initiated. */
  QAPI_DEVICE_INFO_EMM_SERVICE_REQUEST_INITIATED = 5,         /**< Service request initiated. */
  QAPI_DEVICE_INFO_EMM_DEREGISTERED_INITIATED = 6             /**< Deregistered, initiated. */
}emm_state;


/** Valid values for network roaming status (QAPI_DEVICE_INFO_ROAMING_E). */
typedef enum
{
  QAPI_DEVICE_INFO_ROAMING_STATUS_OFF = 0,  /**< Roaming status: OFF. */
  QAPI_DEVICE_INFO_ROAMING_STATUS_ON = 1    /**< Roaming status: ON. */
}roaming_info;


/** Valid value for sim state (QAPI_DEVICE_INFO_SIM_STATE_E). */
typedef enum
{
  QAPI_DEVICE_INFO_SIM_STATE_UNKNOWN = 0x00,            /**< Unknown. */
  QAPI_DEVICE_INFO_SIM_STATE_DETECTED = 0x01,           /**< Detected. */
  QAPI_DEVICE_INFO_SIM_STATE_PIN1_OR_UPIN_REQ = 0x02,   /**< PIN1 or UPIN is required. */
  QAPI_DEVICE_INFO_SIM_STATE_PUK1_OR_PUK_REQ = 0x03,    /**< PUK1 or PUK for UPIN is required. */
  QAPI_DEVICE_INFO_SIM_STATE_PERSON_CHECK_REQ = 0x04,   /**< Personalization state must be checked. */
  QAPI_DEVICE_INFO_SIM_STATE_PIN1_PERM_BLOCKED  = 0x05, /**< PIN1 is blocked. */ 
  QAPI_DEVICE_INFO_SIM_STATE_ILLEGAL = 0x06,            /**< Illegal. */
  QAPI_DEVICE_INFO_SIM_STATE_READY = 0x07               /**< Ready. */
}sim_state;

/** @addtogroup qapi_device_info
@{ */

/** Device information types. */
typedef enum {

/** 
  Response type: Return value of the Device ID as specified in the enum qapi_Device_Info_Type_t.
  Operation: Query      -  Queries the device information using the API qapi_Device_Info_Get().
             Indication -  Sets the device information callback, registering for various indications 
                           using the API qapi_Device_Info_Set_Callback().
 */

 QAPI_DEVICE_INFO_BUILD_ID_E,
 /**< Device BUILD_ID. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_IMEI_E,
 /**< Device IMEI. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_IMSI_E,
 /**< UIM IMSI. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_OS_VERSION_E,
 /**< Device OS version. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_MANUFACTURER_E,
 /**< Device manufacturer. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_MODEL_ID_E,
 /**< Device model ID. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_BATTERY_STATUS_E,
 /**< Device battery status. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEVICE_INFO_BATTERY_PERCENTAGE_E,
 /**< Device battery percentage. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEIVCE_INFO_TIME_ZONE_E,
 /**< Device time zone. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEIVCE_INFO_ICCID_E,
 /**< Device ICCID. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_4G_SIG_STRENGTH_E,
 /**< Network signal strength. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query and Indication. */

 QAPI_DEVICE_INFO_BASE_STATION_ID_E,
 /**< Network base station ID. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEVICE_INFO_MCC_E,
 /**< Network MCC. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query and Indication. */

 QAPI_DEVICE_INFO_MNC_E,
 /**< Network MNC. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query and Indication. */

 QAPI_DEVICE_INFO_SERVICE_STATE_E,
 /**< Network service status. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEVICE_INFO_MDN_E,
 /**< Device MDN. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_TAC_E,
 /**< Network tracking area code. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEVICE_INFO_CELL_ID_E,
 /**< Network cell ID. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query and Indication. */

 QAPI_DEVICE_INFO_RCCS_E,
 /**< Network RRC state. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEVICE_INFO_EMMS_E,
 /**< Network EMM state. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 DEPRACATED1,
 /* Information to keep enum numbering consistent. */

 QAPI_DEVICE_INFO_SERVING_PCI_E,
 /**< Network serving cell PCI. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEVICE_INFO_SERVING_RSRQ_E,
 /**< Serving cell RSRQ. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEVICE_INFO_SERVING_EARFCN_E,
 /**< Serving cell EARFCN. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 DEPRACATED2,
 /** <Info to keep numbering of enum same> */
 DEPRACATED3,
 /** <Info to keep numbering of enum same> */
 DEPRACATED4,
 /** <Info to keep numbering of enum same> */
 DEPRACATED5,
 /** <Info to keep numbering of enum same> */
 DEPRACATED6,
 /** <Info to keep numbering of enum same> */
 QAPI_DEVICE_INFO_NETWORK_IND_E,
 /**< Network indication. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query and Indication. */

 QAPI_DEVICE_INFO_ROAMING_E,
 /**< Roaming status. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query and Indication. */

 QAPI_DEVICE_INFO_LAST_POWER_ON_E,
 /**< Last power on time. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_CHIPID_STRING_E,
 /**< Chipset name. Response type:QAPI_DEVICE_INFO_TYPE_BUFFER_E. Operation: Query. */

 QAPI_DEVICE_INFO_APN_PROFILE_INDEX_E,
 /**< APN profile index. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

 QAPI_DEVICE_INFO_SIM_STATE_E
 /**< SIM state. Response type:QAPI_DEVICE_INFO_TYPE_INTEGER_E. Operation: Query. */

} qapi_Device_Info_ID_t;

/** Device information response types. */
typedef enum {
 QAPI_DEVICE_INFO_TYPE_BOOLEAN_E,
 /**< Response type is Boolean. */
 QAPI_DEVICE_INFO_TYPE_INTEGER_E,
 /**< Response type is integer. */
 QAPI_DEVICE_INFO_TYPE_BUFFER_E,
 /**< Response type is buffer. */
} qapi_Device_Info_Type_t;

/*========================================================
 * Device Info structure                               *
 ========================================================*/

/** Maximum size of #qapi_Device_Info_t valuebuf. */
#define QAPI_DEVICE_INFO_BUF_SIZE 128

/**
 @brief QAPI device information structure.
*/
typedef struct {
  qapi_Device_Info_ID_t id; /**< Required information ID. */
  qapi_Device_Info_Type_t info_type; /**< Response type. */
  union {
    struct {
      char buf[QAPI_DEVICE_INFO_BUF_SIZE]; /**< Response buffer. */
      uint32_t len; /**< Length of the response string. @newpagetable */
    }valuebuf;    /**< Union of buffer values. */
    int valueint; /**< Response integer value. */
    bool valuebool; /**< Response Boolean value.*/
  }u;
}qapi_Device_Info_t;

#ifdef  QAPI_TXM_MODULE

#define qapi_Device_Info_Init()                 ((qapi_Status_t)  (_txm_module_system_call12)(TXM_QAPI_DEVICE_INFO_INIT,    (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0))
#define qapi_Device_Info_Get(a, b)              ((qapi_Status_t)  (_txm_module_system_call12)(TXM_QAPI_DEVICE_INFO_GET,     (ULONG) a, (ULONG) b, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0))
#define qapi_Device_Info_Release()              ((qapi_Status_t)  (_txm_module_system_call12)(TXM_QAPI_DEVICE_INFO_RELEASE, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0))
#define qapi_Device_Info_Set_Callback(a, b)     ((qapi_Status_t)  (_txm_module_system_call12)(TXM_QAPI_DEVICE_INFO_SET_CB,  (ULONG) a, (ULONG) b, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0))
#define qapi_Device_Info_Reset()                  ((qapi_Status_t)  (_txm_module_system_call12)(TXM_QAPI_DEVICE_INFO_RESET,   (ULONG)0, (ULONG)0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0, (ULONG) 0))
#else

UINT qapi_Device_Info_Handler(UINT id, UINT a, UINT b, UINT c, UINT d, UINT e, UINT f, UINT g, UINT h, UINT i, UINT j, UINT k, UINT l);

/*====================================================
 * QAPI Device Info Callback                         *
 ====================================================*/
typedef void (*qapi_Device_Info_Callback_t)(const qapi_Device_Info_t *info);

/*=======================================================
 * QAPI Device Info APIs                             *
 =======================================================*/

/**
 * @brief Initializes the device information context.
 *
 * This function must be called before invoking other qapi_Device_Info APIs.
 *
 * @return QAPI_OK on success, QAPI_ERROR on failure.
 */
qapi_Status_t qapi_Device_Info_Init(void);

/**
 * @brief Gets the device information for specified ID.
 *
 * @param[in] id     Information ID.
 * @param[out] info  Information received for the specified ID.
 *
 * @return QAPI_OK on success, QAPI_ERROR on failure.
 *
 * @dependencies
 * Before calling this API, qapi_Device_Info_Init() must have been called.
 */
qapi_Status_t qapi_Device_Info_Get(qapi_Device_Info_ID_t id, qapi_Device_Info_t *info);

/**
 * @brief Sets a device information callback.
 *
 * @param[in] id        Information ID.
 * @param[in] callback  Callback to be registered.
 *
 * @return QAPI_OK on success, QAPI_ERROR on failure.
 *
 * @dependencies
 * Before calling this API, qapi_Device_Info_Init() must have been called.
 */
qapi_Status_t qapi_Device_Info_Set_Callback(qapi_Device_Info_ID_t id, qapi_Device_Info_Callback_t callback);

/**
 * @brief Releases the device information context.
 *
 * @return QAPI_OK on success, QAPI_ERROR on failure.
 *
 * @dependencies
 * Before calling this API, qapi_Device_Info_Init() must have been called.
 */
qapi_Status_t qapi_Device_Info_Release(void);

/**
 * @brief Resets the device.
 *
 * @return QAPI_OK on success, QAPI_ERROR on failure.
 */
qapi_Status_t qapi_Device_Info_Reset(void);

#endif

/** @} */ /* end_addtogroup qapi_device_info */

#endif
