#ifndef __QAPI_QUECTEL_H__
#define __QAPI_QUECTEL_H__

#include "tx_api.h"
#include "qapi_status.h"
#include "qapi_txm_base.h"
#include "qapi_adc_types.h"
#include "qapi/qapi_dss.h"

/********************* Quectel OEM error number defines **********************/
typedef enum{
	QAPI_QT_ERR_OK = 0x1000,
	QAPI_QT_ERR_NORMAL_FAIL,
	QAPI_QT_ERR_INVALID_PARAM,
	QAPI_QT_ERR_OPERATE_FAILED,
	QAPI_QT_ERR_NO_MEM,
	QAPI_QT_ERR_PORT_ID,
	QAPI_QT_ERR_STREAM_ID,
	QAPI_QT_ERR_LOC_STOPPED,
	QAPI_QT_ERR_LOC_STARTED,

	/*  end  */
	QAPI_QT_ERR_END = 0x2000
}QAPI_QT_ERR_NUM;


/********************* Quectel Release Information **********************/
typedef struct{
	char build_time[32];
	char build_date[32];
	char rel_time[32];
	char rel_date[32];
	char sw_version[128];
	char ql_sw_ver[64];
	char ql_sw_ver_sub[16];
}qapi_QT_Rel_Info;

/********************* Quectel OEM defines **********************/

typedef enum
{
	QAPI_FATAL_ERR_RESET = 0,
	QAPI_FATAL_ERR_SAHARA = 1,
	QAPI_FATAL_ERR_MAX
}QAPI_FATAL_ERR_MODE;

/* --- ATC Pipe --- */
typedef enum qapi_at_port_status_e
{
	QAPI_PORT_OK = 0,
	QAPI_PORT_ERROR,
	QAPI_PORT_INV_ARG,
	QAPI_PORT_ERROR_SIO,
	QAPI_PORT_OPEN_FAILURE,
	QAPI_PORT_MAX
} qapi_at_port_status_t;

typedef enum qpi_at_port_e
{
	QAPI_AT_PORT_0 = 0,
	QAPI_AT_PORT_1,
	QAPI_AT_PORT_2,	//Reserved
	QAPI_AT_PORT_3,	//Reserved

	QAPI_AT_PORT_MAX
} qapi_at_port_t;

typedef struct qapi_atc_pipe_data_s
{
	char   data[2048];
	int    len;
} qapi_at_pipe_data_t; 

typedef signed short int2;

typedef int2	qapi_at_stream_id_t;
/*
@func
  qapi_at_resp_func_cb_t
@brief
  A callback funtion to register when open internal ATC pipe.
@param[out]
  data	- Pointer, to store the data retured from modem side, like AT command executed result or raw data.
*/
typedef void  (*qapi_at_resp_func_cb_t)(qapi_at_pipe_data_t *data);	//response callback

/********** 	GNSS START 	**********/
typedef uint64_t qtLocEventRegMaskT;
#define QT_LOC_EVENT_MASK_POSITION_REPORT 		((qtLocEventRegMaskT)0x00000001ull)
#define QT_LOC_EVENT_MASK_GNSS_SV_INFO 			((qtLocEventRegMaskT)0x00000002ull)
#define QT_LOC_EVENT_MASK_NMEA 					((qtLocEventRegMaskT)0x00000004ull)

/*
@func
  qapi_QT_Loc_CB_t
@brief
  A callback funtion to register when Start GNSS engine.
@param[out]
  nmea_data	- Pointer, to store NMEA sentenses returned by GNSS engine.
*/
typedef void (*qapi_QT_Loc_CB_t)(char *nmea_data);	//response callback

/********** 	GNSS END 	**********/

/********** 	Fota START 	**********/
/*
@func
  qapi_Fota_Http_dl_CB_t
@brief
  A callback funtion to register when Start FOTA HTTP download.
@param[out]
  error_id - Error code.
         0 -- Download successful
       701 -- HTTP(S) unknown error
    others -- unknown error
*/
typedef void(*qapi_Fota_Http_dl_CB_t)(int error_id);

/*
@func
  qapi_Fota_Http_dl_CB_t
@brief
  A callback funtion to register when Start FOTA upgrade process.
@param[out]
  error_id - Error code.
      0 -- upgrade success 
    504 -- firmware upgrade failed
    505 -- upgrade package not exist   
    506 -- upgrade package check failed
*/
typedef void (*qapi_QT_Fota_Response_CB_t)(short int error_id);	//response callback


/*
@func
  qapi_Fota_Http_dl_CB_t
@brief
  A callback funtion to register when Start FOTA upgrade process, used to report the upgrade progress.
@param[out]
  phase - Status of the upgrade.
    0 -- upgrading period
    1 -- restoring period
  Note - The fota upgrading start with upgrading period.
		 When upgrading progress reach to 100%, it will become to restoring period.
		 When restoring progress reach to 100%, It means that upgrading is successful.
		 Then the module should be restarted.
  percent - Indicator of upgrade process.
            range, from 0 to 100.
*/
typedef void (*qapi_QT_Fota_Upgrade_Progress_CB_t)(unsigned char phase, unsigned char percent);	//response callback

/********** 	Fota END 	**********/


/********************* Quectel OEM ID defines **********************/
/* @Note: Each new OEM ID needs to be defined in this enum */
typedef enum
{
	TXM_OEM_API_QT_BASE = TXM_QC_OEM_API_BASE,	/**<-Quectel TXM API IDs Start with 0x20000->*/
	TXM_OEM_API_QT_REL_INFO = TXM_OEM_API_QT_BASE+1,
	TXM_OEM_API_QT_RESET_DEVICE = TXM_OEM_API_QT_BASE+2,

	/** ATC PIPE TXM ID */
	TXM_OEM_API_AT_PORT_OPEN = TXM_OEM_API_QT_BASE+3,
	TXM_OEM_API_AT_SEND = TXM_OEM_API_QT_BASE+4,
	TXM_OEM_API_AT_SEND_HEXBYTE = TXM_OEM_API_QT_BASE+5,
	TXM_OEM_API_AT_SEND_GENERAL = TXM_OEM_API_QT_BASE+6,
	TXM_OEM_API_AT_PORT_CLOSE = TXM_OEM_API_QT_BASE+7,

	/** ADC TXM ID */
	TXM_OEM_API_QT_ADC_READ = TXM_OEM_API_QT_BASE+8,

	/** GNSS TXM ID */
	TXM_OEM_API_QT_LOC_START = TXM_OEM_API_QT_BASE+9,
	TXM_OEM_API_QT_LOC_STOP = TXM_OEM_API_QT_BASE+10,

    /** FOTA TXM ID */
    TXM_OEM_API_QT_FOTA_REMOVE_PACKAGE = TXM_OEM_API_QT_BASE+11,
    TXM_OEM_API_QT_FOTA_HTTP_DL_START = TXM_OEM_API_QT_BASE+12,
    TXM_OEM_API_QT_FOTA_UPDATE_START = TXM_OEM_API_QT_BASE+13,

	/** RAI TXM ID*/
	TXM_OEM_API_QT_RAI = TXM_OEM_API_QT_BASE+14,

	TXM_OEM_API_QT_SAHARA_GET = TXM_OEM_API_QT_BASE+15,
	TXM_OEM_API_QT_SAHARA_SET = TXM_OEM_API_QT_BASE+16,

	/** PSM CORE CONFIG TXM ID*/
	TXM_OEM_API_QT_PSM_SRV_CFG_SET = TXM_OEM_API_QT_BASE+17,
	TXM_OEM_API_QT_PSM_SRV_CFG_GET = TXM_OEM_API_QT_BASE+18,

	/** SHUTDOWN TXM ID*/
	TXM_OEM_API_QT_SHUTDOWN_DEVICE = TXM_OEM_API_QT_BASE+19,

	TXM_OEM_API_QT_REG_WDOG = TXM_OEM_API_QT_BASE+20,
	TXM_OEM_API_QT_KICK_WDOG = TXM_OEM_API_QT_BASE+21,
	TXM_OEM_API_QT_STOP_WDOG = TXM_OEM_API_QT_BASE+22,
	TXM_OEM_API_QT_USBMODE_SET = TXM_OEM_API_QT_BASE+23,

	/* USB EVENT */
	TXM_OEM_API_QT_GET_USB_EVENT = TXM_OEM_API_QT_BASE+24,
	TXM_OEM_API_QT_PWRKEY_EVENT_SET = TXM_OEM_API_QT_BASE+25,
	TXM_OEM_API_QT_PWRKEY_EVENT_GET = TXM_OEM_API_QT_BASE+26,

	/* GET RANDOM DATE */
	TXM_OEM_API_QT_PRNG_DATA_GET = TXM_OEM_API_QT_BASE+27,
	TXM_OEM_API_QT_START_UP_TYPE_GET = TXM_OEM_API_QT_BASE+28,

    /** FOTA EXTENDED TXM ID */
    TXM_OEM_API_QT_FOTA_GET_PACKAGE_INFO = TXM_OEM_API_QT_BASE+29,
    TXM_OEM_API_QT_FOTA_HTTP_DL_CANCEL = TXM_OEM_API_QT_BASE+30,

    /** USB NMEA PORT OPERATION TXM ID */
	TXM_OEM_API_QT_NMEA_SIO_OPEN_API = TXM_OEM_API_QT_BASE+31,
	TXM_OEM_API_QT_NMEA_SIO_CLOSE_API = TXM_OEM_API_QT_BASE+32,
	TXM_OEM_API_QT_NMEA_SIO_TRANSMIT_API = TXM_OEM_API_QT_BASE+33,
	TXM_OEM_API_QT_USB_SWITCH_SET = TXM_OEM_API_QT_BASE+34,
	TXM_OEM_API_QT_USB_SWITCH_GET = TXM_OEM_API_QT_BASE+35,
	TXM_OEM_API_QT_END = TXM_QC_OEM_API_END		/**<-Quectel TXM API IDs End with 0x3000->*/
} TXM_OEM_API_ID_DEFINES_E;

/********************* Quectel OEM QAPI functions **********************/
#ifdef  QAPI_TXM_MODULE  

#define qapi_QT_Welcome(a)			((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_BASE,(ULONG)a, (ULONG)0, (ULONG)0))
#define qapi_QT_Get_Rel_Info(a)		((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_REL_INFO,(ULONG)a, (ULONG)0, (ULONG)0))
#define qapi_QT_Reset_Device(a)		((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_RESET_DEVICE,(ULONG)a, (ULONG)0, (ULONG)0))

#define qapi_QT_Apps_AT_Port_Open(a, b, c)		((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_AT_PORT_OPEN,(ULONG)a, (ULONG)b, (ULONG)c))
#define qapi_QT_Apps_Send_AT(a, b)				((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_AT_SEND,(ULONG)a, (ULONG)b, (ULONG)0))
#define qapi_QT_Apps_Send_AT_HexByte(a, b)		((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_AT_SEND_HEXBYTE,(ULONG)a, (ULONG)b, (ULONG)0))
#define qapi_QT_Apps_Send_AT_General(a, b, c)	((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_AT_SEND_GENERAL,(ULONG)a, (ULONG)b, (ULONG)c))
#define qapi_QT_Apps_AT_Port_Close(a)			((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_AT_PORT_CLOSE,(ULONG)a, (ULONG)0, (ULONG)0))

#define qapi_QT_ADC_Read(a,b)		((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_ADC_READ,(ULONG)a, (ULONG)b, (ULONG)0))
#define qapi_QT_RAI_Set(a,b)						((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_RAI,(ULONG)a, (ULONG)b, (ULONG)0))

/********** 	GNSS START 	**********/
#define qapi_QT_Loc_Start(a,b)		((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_LOC_START,(ULONG)a, (ULONG)b, (ULONG)0))
#define qapi_QT_Loc_Stop()			((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_LOC_STOP,(ULONG)0, (ULONG)0, (ULONG)0))
/********** 	GNSS END 	**********/

/********** 	FOTA START 	**********/
#define qapi_QT_Remove_Fota_Package()              ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_FOTA_REMOVE_PACKAGE,(ULONG)0, (ULONG)0, (ULONG)0)) 
#define qapi_QT_Fota_Http_Download_Start(a,b)      ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_FOTA_HTTP_DL_START,(ULONG)a, (ULONG)b, (ULONG)0))
#define qapi_QT_Fota_Update_Start(a,b)		       ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_FOTA_UPDATE_START,(ULONG)a, (ULONG)b, (ULONG)0))
#define qapi_QT_Fota_Get_Package_Info(a,b)         ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_FOTA_GET_PACKAGE_INFO,(ULONG)a, (ULONG)b, (ULONG)0))
#define qapi_QT_Fota_Http_Download_Cancel()        ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_FOTA_HTTP_DL_CANCEL,(ULONG)0, (ULONG)0, (ULONG)0))
/********** 	FOTA END 	**********/

#define qapi_QT_Sahara_Mode_Get(a)		((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_SAHARA_GET,(ULONG)a, (ULONG)0, (ULONG)0))
#define qapi_QT_Sahara_Mode_Set(a)		((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_SAHARA_SET,(ULONG)a, (ULONG)0, (ULONG)0))


#define qapi_QT_PSM_SrvCfg_Set(a)					((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_PSM_SRV_CFG_SET,(ULONG)a, (ULONG)0, (ULONG)0))
#define qapi_QT_PSM_SrvCfg_Get(a)					((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_PSM_SRV_CFG_GET,(ULONG)a, (ULONG)0, (ULONG)0))

#define qapi_QT_Shutdown_Device()					((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_SHUTDOWN_DEVICE,(ULONG)0, (ULONG)0, (ULONG)0))
#define qapi_QT_Register_Wdog(a,b)					((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_REG_WDOG,(ULONG)a, (ULONG)b, (ULONG)0))
#define qapi_QT_Kick_Wdog(a)					((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_KICK_WDOG,(ULONG)a, (ULONG)0, (ULONG)0))
#define qapi_QT_Stop_Wdog(a)					((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_STOP_WDOG,(ULONG)a, (ULONG)0, (ULONG)0))
#define qapi_QT_USB_Mode_Set(a)					((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_USBMODE_SET,(ULONG)a, (ULONG)0, (ULONG)0))

#define qapi_QT_Get_USB_Event(a)					((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_GET_USB_EVENT,(ULONG)a, (ULONG)0, (ULONG)0))
#define qapi_QT_PWRKEY_Switch_Set(a)             ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_PWRKEY_EVENT_SET,(ULONG)a, (ULONG)0, (ULONG)0))
#define	qapi_QT_PWRKEY_State_Get(a)               ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_PWRKEY_EVENT_GET,(ULONG)a, (ULONG)0, (ULONG)0))
#define	qapi_QT_Random_Data_Get(a,b)               ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_PRNG_DATA_GET,(ULONG)a, (ULONG)b, (ULONG)0))
#define qapi_QT_Startup_Type_Get(a)  			((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_START_UP_TYPE_GET,(ULONG)a, (ULONG)0,(ULONG)0))
#define qapi_QT_USB_Sio_Open()				        ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_NMEA_SIO_OPEN_API,(ULONG)0, (ULONG)0,(ULONG)0))
#define qapi_QT_USB_Sio_Close()				        ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_NMEA_SIO_CLOSE_API,(ULONG)0, (ULONG)0,(ULONG)0))
#define qapi_QT_USB_Sio_Transmit(a)				    ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_NMEA_SIO_TRANSMIT_API,(ULONG)a, (ULONG)0,(ULONG)0))
#define qapi_QT_USB_Switch_Set(a)				    ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_USB_SWITCH_SET,(ULONG)a, (ULONG)0,(ULONG)0))
#define qapi_QT_USB_Switch_Get(a)				    ((UINT)(_txm_module_kernel_call_dispatcher)(TXM_OEM_API_QT_USB_SWITCH_GET,(ULONG)a, (ULONG)0,(ULONG)0))
#else

/*
@func
  qapi_QT_Welcome
@brief
  An example for QAPI public.
*/
qapi_Status_t qapi_QT_Welcome(char *welcome);

/*
@func
  qapi_QT_Get_Rel_Info
@brief
  Get apps release information.
@param[out]
  rel_info - Firmware release information.
*/
qapi_Status_t qapi_QT_Get_Rel_Info(qapi_QT_Rel_Info *rel_info);

/*
@func
  qapi_QT_Reset_Device
@brief
  reset module. 
@param
  mode -- reserved.
*/
qapi_Status_t qapi_QT_Reset_Device(uint16_t mode);

/*
@func
  qapi_QT_Shutdown_Device
@brief
  Shutdown module. 
*/
qapi_Status_t qapi_QT_Shutdown_Device(void);

/*
@func
  qapi_QT_Apps_AT_Port_Open
@brief
  Open Apps AT command port and register a callback. 
*/
qapi_Status_t qapi_QT_Apps_AT_Port_Open(qapi_at_port_t port_id, qapi_at_stream_id_t *stream_id, qapi_at_resp_func_cb_t cb);

/*
@func
  qapi_QT_Apps_Send_AT
@brief
  Send AT command in specifial stream port id. 
*/
qapi_Status_t qapi_QT_Apps_Send_AT(qapi_at_stream_id_t stream_id, const char *command_name);

/*
@func
  qapi_QT_Apps_Send_AT_HexByte
@brief
  Send string in specifial stream port id with Hex format.
*/
qapi_Status_t qapi_QT_Apps_Send_AT_HexByte(qapi_at_stream_id_t stream_id, const char *hex_str);

/*
@func
  qapi_QT_Apps_Send_AT_General
@brief
  Send typeless data in secifial stream port id. Caller can tranmit any type of data through stream port. And
  response will be notified in callback which registed in qapi_QT_Apps_AT_Port_Open(). 
*/
qapi_Status_t qapi_QT_Apps_Send_AT_General(qapi_at_stream_id_t stream_id, void *trans_data, uint32 trans_len);

/*
@func
  qapi_QT_Apps_AT_Port_Close
@brief
  Close AT command port with stream port id. 
*/
void qapi_QT_Apps_AT_Port_Close(qapi_at_stream_id_t stream_id);

/*
@func
  qapi_QT_ADC_Read
@brief
  Read the value of ADC.
@param[in]
  pChn_Name	- Adc Channel name.
@param[out]
  result - Result of related ADC channel.
*/
qapi_Status_t qapi_QT_ADC_Read(const char *pChn_Name,qapi_ADC_Read_Result_t *result);

/*
@func
  qapi_QT_Loc_Start
@brief
  Start GNSS Location.
@param[in]
  evt_mask - Event Registration Mask.
    QT_LOC_EVENT_MASK_POSITION_REPORT -- The control point must enable this mask
    									 to receive position report event indications. 
    QT_LOC_EVENT_MASK_GNSS_SV_INFO -- The control point must enable this mask to receive satellite 
   								      report event indications. These reports are sent at a 1 Hz rate. 
    QT_LOC_EVENT_MASK_NMEA  --  The control point must enable this mask to receive NMEA reports for 
   							    position and satellites in view. The report is at a 1 Hz rate.
@param[out]
  cb	Callback function. Refer to the description qapi_QT_Loc_CB_t. 
*/
qapi_Status_t qapi_QT_Loc_Start(qtLocEventRegMaskT evt_mask, qapi_QT_Loc_CB_t cb);

/*
@func
  qapi_QT_Loc_Stop
@brief
  Stop GNSS Location.
*/
qapi_Status_t qapi_QT_Loc_Stop(void);

/*
@func
  qapi_QT_Remove_Fota_Package
@brief
  Remove fota package.
*/
qapi_Status_t qapi_QT_Remove_Fota_Package(void);

/*
@func
  qapi_QT_Fota_Http_Download_Start
@brief
  Start Fota package download.
@param[in]
  url - fota package path.
        for example: HTTP://220.180.239.212:8005/BG96_112A_119.zip.
        The url must be started with "http://" or "https://", not case sensitive.
  response_cb - response callback, This callback will come when download failed or success. 
                details please refer to the description of qapi_Fota_Http_dl_CB_t.
*/
qapi_Status_t qapi_QT_Fota_Http_Download_Start(char* url, qapi_Fota_Http_dl_CB_t response_cb);

/*
@func
  qapi_QT_Fota_Update_Start
@brief
  Start Fota Update.
  Note, Before use this interface, the delta firmware should be uploded to the efs.
        The storage path for delta firmware package is /fota/update.zip.
        Both the path and the package name cannot be modified.
        Customers can use the QEFS Explorer tool provided by Quectel to upload the upgrade package
@param [in]
  response_cb - response callback,This callback will come when fota upgrade failed or success.
  upgrade_progress_cb - upgrade progress callback, This callback will come every one percent.
*/
qapi_Status_t qapi_QT_Fota_Update_Start(qapi_QT_Fota_Response_CB_t response_cb, qapi_QT_Fota_Upgrade_Progress_CB_t upgrade_progress_cb);

/*
@func
  qapi_QT_Fota_Get_Package_Info
@brief
  Get the DFOTA Package Information.
@param[out]
  dwl_size  Pointer, size of the downloaded file.
  total_size  Pointer, total size of the package file on the HTTP server.
*/
qapi_Status_t qapi_QT_Fota_Get_Package_Info(uint32 *dwl_size, uint32 *total_size);

/*
@func
  qapi_QT_Fota_Http_Download_Cancel
@brief
  Cancel the current download process.
*/
qapi_Status_t qapi_QT_Fota_Http_Download_Cancel(void);

/*
@func
  qapi_QT_RAI_Set
@brief
  Set the Release Assistant Information indicator.
@param[in]
  hndl - Pointer to data service handle
  rai_type - Type of Release Assistant Information indicator
     	     0 -- No information available
  		     1 -- No further uplink or downlink data transmission
  	    	 2 -- Only a single downlink data transmission and no further uplink data 
*/
qapi_Status_t qapi_QT_RAI_Set(qapi_DSS_Hndl_t hndl, uint8_t rai_type);

/*
@func
  qapi_QT_Sahara_Mode_Get
@brief
  Get the NV Item value of Sahara mode setting.
@param[out]
  mode - Configure options.
    QAPI_FATAL_ERR_RESET -- Reset mode.
    QAPI_FATAL_ERR_SAHARA -- Sahara DUMP mode.
*/
qapi_Status_t qapi_QT_Sahara_Mode_Get(QAPI_FATAL_ERR_MODE *mode);

/*
@func
  qapi_QT_Sahara_Mode_Set
@brief
  Enable or disable module enter sahara dump mode when module crash.
  It will take effect after the module restarted.
@param[in]
  mode - Configure options.
    QAPI_FATAL_ERR_RESET -- Reset mode.
    QAPI_FATAL_ERR_SAHARA -- Sahara DUMP mode.
*/
qapi_Status_t qapi_QT_Sahara_Mode_Set(QAPI_FATAL_ERR_MODE mode);

/*
@func
  qapi_QT_PSM_SrvCfg_Set
@brief
  Set PSM core server configuration
@param[in]
  gpio_enable - Configure options.
    0 -- Disable GPIO(PSM_IND) indicator.
    1 -- Enable GPIO(PSM_IND) indicator.
*/
qapi_Status_t qapi_QT_PSM_SrvCfg_Set(int gpio_enable);

/*
@func
  qapi_QT_PSM_SrvCfg_Get
@brief
  Get PSM core server configuration
@param[in]
  gpio_enable - Configure options.
    0 -- GPIO(PSM_IND) indicator disabled.
    1 -- GPIO(PSM_IND) indicator Enabled.
*/
qapi_Status_t qapi_QT_PSM_SrvCfg_Get(int *gpio_enable);

/*
@func
  qapi_QT_Register_dog
@brief
  Dynamic method for tasks to register for dog services
@param [in]
  time  starvation detection threshhold in milliseconds 
@param [out]
  dog_id  for feed watchdog function use
*/
qapi_Status_t qapi_QT_Register_Wdog( uint32 time,uint32 *dog_id);

/*
@func
  qapi_QT_kick_watchdog
@brief
  Report to the Watchdog task, so that it can tell that the monitored
  task is still functioning properly. 
@param [in]  
  id  register Wdog function return dog_id
*/
qapi_Status_t qapi_QT_Kick_Wdog(uint32 id);

/*
@func
  qapi_QT_stop_watchdog
@brief
  Set the sotftware dog_enable on/off
@param [in]
  mode enable or disable watchdog
       1  disable all watchdog
       0  able all watchdog    
*/
qapi_Status_t qapi_QT_Stop_Wdog(boolean mode);

/*
@func
  qapi_QT_USB_Mode_Set
@brief
  Set the sw usb on/off
@param [in]
  mode enable or disable the usb
       1  disable the usb
       0  able the usb   
*/
qapi_Status_t qapi_QT_USB_Mode_Set(boolean mode);

/*
@func
  qapi_QT_Get_USB_Event
@brief
  Get USB events : 
    0 : DEVICE CONNECT
    1 : DEVICE DISCONNECT
    2 : DEVICE SUSPEND
    3 : DEVICE RESUME
    4 : DEVICE RESUME COMPLETED
    5 : DEVICE REMOTE WAKEUP
    6 : DEVICE CONFIGURED
    7 : DEVICE UNCONFIGURED
    8 : DEVICE RESET
    9 : DEVICE SPEED CHANGE
@param[in]
  usb_evt : pointer, store the USB Event.
*/
qapi_Status_t qapi_QT_Get_USB_Event(uint32 *usb_evt);

/*
@func
  qapi_QT_PWRKEY_Switch_Set
@brief
  Set the sw PWRKEY enable/disable
@param [in]
  val enable or disable the PWRKEY
       1  disable the PWRKEY When module is turn on
       0  enable the PWRKEY
*/
qapi_Status_t qapi_QT_PWRKEY_Switch_Set(uint32 mode);

/*
@func
  qapi_QT_PWRKEY_State_Get
@brief
  Get PWRKEY state

*/
qapi_Status_t qapi_QT_PWRKEY_State_Get(uint32 *val);

/*
@func
  qapi_QT_Random_Data_Get
@brief
  Gets the hardware random number
@param [in]
  prng_size  get the length of random Number.Parameters range of the support:1,2,4...512 
@param [out]
  prng_data  Stores the generated random Number,the size of the data returned depends on prng_size
*/
qapi_Status_t qapi_QT_Random_Data_Get(uint16 prng_size, uint8* prng_data);

/*
@func
  qapi_QT_Startup_Type_Get
@brief
  Gets the Module startup reason type
@param [out]
  Val  Return the Module startup reason type value
*/
qapi_Status_t qapi_QT_Startup_Type_Get(uint16 *val);
/*
quectel Alfred 20190505
ӽӿڣSDKnmea USBʹ
*/
/*
@func
  qapi_QT_USB_Sio_Open
@brief
  Open NMEA USB port for output user application log.
*/
qapi_Status_t qapi_QT_USB_Sio_Open(void);

/*
@func
  qapi_QT_USB_Sio_Close
@brief
  Close NMEA USB port.
*/
qapi_Status_t qapi_QT_USB_Sio_Close(void);

/*
@func
  qapi_QT_USB_Sio_Transmit
@brief
  Output user application log.
@param [in]
  Pointer, point to the buffer which store the user application log.
*/
qapi_Status_t qapi_QT_USB_Sio_Transmit(char *log);

/*
@func
  qapi_QT_USB_Switch_Set
@brief
  Set a global flag to prevent changing USB mode setting by the AT command AT+QCFGEXT="disusb".
@param [in]
  Enable or disable configuration of USB functions by the AT command
       1  Enable configuration of USB functions by the AT command
       0  Disable configuration of USB functions by the AT command     
*/
qapi_Status_t qapi_QT_USB_Switch_Set(boolean mode);

/*
@func
  qapi_QT_USB_Switch_Get
@brief
  Get the configuration by qapi_QT_USB_Switch_Set.
@param [out]
  get the global flag value   
*/
qapi_Status_t qapi_QT_USB_Switch_Get(uint8* mode);

#endif

#endif /*__QAPI_QUECTEL_H__*/
