/******************************************************************************
*@file    example_i2c.c
*@brief   example of i2c
*
*  ---------------------------------------------------------------------------
*
*  Copyright (c) 2018 Quectel Technologies, Inc.
*  All Rights Reserved.
*  Confidential and Proprietary - Quectel Technologies, Inc.
*  ---------------------------------------------------------------------------
*******************************************************************************/
#if defined(__EXAMPLE_ANALOGI2C__)
#include "txm_module.h"
#include "qapi_diag.h"
#include "qapi_timer.h"
#include "qapi_uart.h"
#include "quectel_utils.h"
#include "quectel_uart_apis.h"
#include "qapi_fs_types.h"
#include "qapi_fs.h"
#include "example_i2c.h"
#include "qapi_i2c_master.h"
#include "qapi_tlmm.h"
#include "qapi_timer.h"
#include "qapi_diag.h"
#include "quectel_utils.h"
#include "quectel_gpio.h"

/**************************************************************************
*                                 DEFINE
***************************************************************************/
#define QUEC_I2C_UART_DBG
#ifdef QUEC_I2C_UART_DBG
#define I2C_UART_DBG(...)	\
{\
	i2c_uart_debug_print(__VA_ARGS__);	\
}
#else
#define I2C_UART_DBG(...)
#endif

/**************************************************************************
*                                 GLOBAL
***************************************************************************/
/* Uart Dbg */
qapi_UART_Handle_t i2c_dbg_uart_handler;
static char quec_dbg_buffer[128];

void i2c_uart_dbg_init(void)
{
	qapi_Status_t status;
	qapi_UART_Open_Config_t uart_cfg;

	uart_cfg.baud_Rate			= 115200;
	uart_cfg.enable_Flow_Ctrl	= QAPI_FCTL_OFF_E;
	uart_cfg.bits_Per_Char		= QAPI_UART_8_BITS_PER_CHAR_E;
	uart_cfg.enable_Loopback	= 0;
	uart_cfg.num_Stop_Bits		= QAPI_UART_1_0_STOP_BITS_E;
	uart_cfg.parity_Mode		= QAPI_UART_NO_PARITY_E;
	uart_cfg.rx_CB_ISR			= NULL;
	uart_cfg.tx_CB_ISR			= NULL;

	status = qapi_UART_Open(&i2c_dbg_uart_handler, QAPI_UART_PORT_002_E, &uart_cfg);
	if (QAPI_OK != status)
	{
		return;
	}
	status = qapi_UART_Power_On(i2c_dbg_uart_handler);
}

void i2c_uart_debug_print(const char* fmt, ...) 
{
	va_list arg_list;

	va_start(arg_list, fmt);
    vsnprintf((char *)(quec_dbg_buffer), sizeof(quec_dbg_buffer), (char *)fmt, arg_list);
    va_end(arg_list);
		
	qapi_UART_Transmit(i2c_dbg_uart_handler, quec_dbg_buffer, strlen(quec_dbg_buffer), NULL);
	qapi_UART_Transmit(i2c_dbg_uart_handler, "\r\n", strlen("\r\n"), NULL);
	qapi_Timer_Sleep(10, QAPI_TIMER_UNIT_MSEC, true);
}


/**************************************************************************
*                                 DEFINE
***************************************************************************/

qapi_GPIO_Value_t GPIO_ReadInputDataBit_sda(void);
void SDA_GPIO_out_InitIO(void);
void SDA_GPIO_IN_InitIO(void);
void SCL_GPIO_InitIO(void);


GPIO_MAP_TBL gpio_map_tbl[PIN_E_GPIO_MAX] = {
/* PIN NUM,     PIN NAME,    GPIO ID  GPIO FUNC */
	{  40, 		"scl",  	19, 	 0},
	{  41,		"sda",		18, 	 0},		
};

qapi_TLMM_Config_t tlmm_config[2];
qapi_GPIO_ID_t     gpio_id_tbl[2];
qapi_TLMM_Config_t tlmm_config[2];



#define SDA         1
#define SCL         0


#define SIMSET_I2C_CLK_OUTPUT()		   	     SCL_GPIO_InitIO()
#define SIMSET_I2C_CLK_HIGH()			     qapi_TLMM_Drive_Gpio(gpio_id_tbl[SCL], gpio_map_tbl[SCL].gpio_id, QAPI_GPIO_HIGH_VALUE_E);
#define SIMSET_I2C_CLK_LOW()		         qapi_TLMM_Drive_Gpio(gpio_id_tbl[SCL], gpio_map_tbl[SCL].gpio_id, QAPI_GPIO_LOW_VALUE_E);
#define SIMSET_I2C_DATA_OUTPUT()			 SDA_GPIO_out_InitIO()
#define SIMSET_I2C_DATA_INPUT()		   	     SDA_GPIO_IN_InitIO()
#define SIMSET_I2C_DATA_HIGH()			     qapi_TLMM_Drive_Gpio(gpio_id_tbl[SDA], gpio_map_tbl[SDA].gpio_id, QAPI_GPIO_HIGH_VALUE_E);
#define SIMSET_I2C_DATA_LOW()			     qapi_TLMM_Drive_Gpio(gpio_id_tbl[SDA], gpio_map_tbl[SDA].gpio_id, QAPI_GPIO_LOW_VALUE_E);
#define SIMGET_I2C_DATA_BIT()			     GPIO_ReadInputDataBit_sda()


static uint32 savedMask;


#define KAL_TRUE         1
#define KAL_FALSE        0


#define I2C_SCL            (1<<1)                   // bit  1  
#define I2C_SDA            (1<<2)                   // bit  2  


#define I2C_INPUT         QAPI_GPIO_INPUT_E         //in_put
#define I2C_OUTPUT        QAPI_GPIO_OUTPUT_E        //out_put

#define I2C_SET_OUT       QAPI_GPIO_HIGH_VALUE_E	//high
#define I2C_CLEAR_OUT     QAPI_GPIO_LOW_VALUE_E		//low




#define I2C_DELAY       5000 //10  us
#define I2C_DELAYREAD1	30
#define I2C_DELAYREAD2	20




void gpio_config(MODULE_PIN_ENUM m_pin,
				 qapi_GPIO_Direction_t gpio_dir,
				 qapi_GPIO_Pull_t gpio_pull,
				 qapi_GPIO_Drive_t gpio_drive
				 )
{
	qapi_Status_t status = QAPI_OK;

	tlmm_config[m_pin].pin   = gpio_map_tbl[m_pin].gpio_id;
	tlmm_config[m_pin].func  = gpio_map_tbl[m_pin].gpio_func;
	tlmm_config[m_pin].dir   = gpio_dir;
	tlmm_config[m_pin].pull  = gpio_pull;
	tlmm_config[m_pin].drive = gpio_drive;

	// the default here
	status = qapi_TLMM_Get_Gpio_ID(&tlmm_config[m_pin], &gpio_id_tbl[m_pin]);
	if (status == QAPI_OK)
	{
		status = qapi_TLMM_Config_Gpio(gpio_id_tbl[m_pin], &tlmm_config[m_pin]);
	
		if (status != QAPI_OK)
		{
			IOT_DEBUG("QT# gpio_config failed");
		}
	}
}

qapi_GPIO_Value_t GPIO_ReadInputDataBit_sda(void)
{
	qapi_GPIO_Value_t level; 
	qapi_TLMM_Read_Gpio(gpio_id_tbl[SDA], gpio_map_tbl[SDA].gpio_id, &level);

   return level ;
}


void SDA_GPIO_out_InitIO(void)
{
	qapi_TLMM_Release_Gpio_ID(&tlmm_config[SDA], gpio_id_tbl[SDA]);
    gpio_config(SDA, QAPI_GPIO_OUTPUT_E, QAPI_GPIO_NO_PULL_E, QAPI_GPIO_2MA_E);

}
void SDA_GPIO_IN_InitIO(void)
{
	qapi_TLMM_Release_Gpio_ID(&tlmm_config[SDA], gpio_id_tbl[SDA]);
    gpio_config(SDA, QAPI_GPIO_INPUT_E, QAPI_GPIO_NO_PULL_E, QAPI_GPIO_2MA_E);

}


void SCL_GPIO_InitIO(void)
{
	
    qapi_TLMM_Release_Gpio_ID(&tlmm_config[SCL], gpio_id_tbl[SCL]); 
    gpio_config(SCL, QAPI_GPIO_OUTPUT_E, QAPI_GPIO_NO_PULL_E, QAPI_GPIO_2MA_E);	

}

static void  SetI2CDelay(void)
{
  int   j;
  for(j=0; j < I2C_DELAY ;j++);  
}

static void  SetI2CDelayE(int delay)
{
  int   j;
  
  for(j=0; j < delay ;j++);   
}

static void SetI2CLongDelay(void)
{
  int i = 0;
  for(i = 0; i < 5; i++)
  {
    SetI2CDelay();
  }
}


static void  SetI2CDir(uint8 mask, uint8 config)
{
  if (config == I2C_OUTPUT) 
  {
    if(mask == I2C_SDA)
    {
	SIMSET_I2C_DATA_OUTPUT();
    }
  }
  else if(config == I2C_INPUT)
  {
    if(mask == I2C_SDA)
    {
	SIMSET_I2C_DATA_INPUT();
    }
  }
}


static void  SetI2COutput(uint8 mask, uint8 state)
{
  if (state == I2C_CLEAR_OUT) 
  {
    if(mask & I2C_SDA)
    {
	SIMSET_I2C_DATA_LOW();
    }
    if(mask & I2C_SCL)
    {
	  SIMSET_I2C_CLK_LOW();
    }
  }
  else if(state == I2C_SET_OUT)
  {
    if(mask & I2C_SDA)
    {
	SIMSET_I2C_DATA_HIGH();
    }
    if(mask & I2C_SCL)
    {
	SIMSET_I2C_CLK_HIGH();
    }
  }
}


static uint8  GetI2CSDA()
{
  uint8 ret;
  ret = SIMGET_I2C_DATA_BIT();
  return ret;
}


static void  StartI2C()
{  

    SetI2COutput(I2C_SDA, I2C_SET_OUT);
    SetI2COutput(I2C_SCL, I2C_SET_OUT);
    SetI2CDelay();
    SetI2COutput( I2C_SDA, I2C_CLEAR_OUT);
    SetI2CDelay();    
	
}



static void  StopI2C()
{
    SetI2CDir( I2C_SDA, I2C_OUTPUT);
    SetI2COutput(I2C_SDA, I2C_CLEAR_OUT);
    SetI2CDelay();
    SetI2COutput(I2C_SCL, I2C_SET_OUT);
    SetI2CDelay();
    SetI2COutput(I2C_SDA, I2C_SET_OUT);
    SetI2CDelay(); 
}


static void  WriteI2CData(uint8 data)
{
    uint8 i;
    uint8  cShift;
	
    SetI2COutput( I2C_SCL, I2C_CLEAR_OUT);
	
    SetI2CDelay();

    SetI2CDir( I2C_SDA, I2C_OUTPUT);
    SetI2CDelay();
    cShift = 0x80;

  for(i = 0; i < 4; i++) 
  {
	
	  if(data & cShift) 
	  {
		  SetI2COutput( I2C_SDA, I2C_SET_OUT);
		  SetI2COutput( I2C_SCL, I2C_CLEAR_OUT);
	  }
	  else 
	  {
		  SetI2COutput( I2C_SDA, I2C_CLEAR_OUT);
		  SetI2COutput( I2C_SCL, I2C_CLEAR_OUT);
	  }
	

	  SetI2CDelay();
	  SetI2COutput( I2C_SCL, I2C_SET_OUT);
	  SetI2CDelay();
	
	  SetI2COutput( I2C_SCL, I2C_CLEAR_OUT);
	  SetI2COutput( I2C_SDA, I2C_CLEAR_OUT);
	
	  SetI2CDelay();
	  cShift >>= 1;
  }

 for(i = 4; i < 8; i++) 
  {	 
	  if(data & cShift) 
	  {
		  SetI2COutput( I2C_SDA, I2C_SET_OUT);
		  SetI2COutput( I2C_SCL, I2C_CLEAR_OUT);
	  }
	  else 
	  {
		  SetI2COutput( I2C_SDA, I2C_CLEAR_OUT);
		  SetI2COutput( I2C_SCL, I2C_CLEAR_OUT);
	  }
	  SetI2CDelay();
	  SetI2COutput( I2C_SCL, I2C_SET_OUT);
	  SetI2CDelay(); 
	  SetI2COutput(I2C_SCL, I2C_CLEAR_OUT);
	  SetI2COutput( I2C_SDA, I2C_CLEAR_OUT);
	  if(i==7)
	  {
		  SetI2CDir( I2C_SDA, I2C_INPUT);
	  }
	  SetI2CDelay();
	  cShift >>= 1;
  }

}


static uint8  ReadI2CData()
{
    uint8 i;
    uint8  ReceiveData;
    uint8  bit_val;

    SetI2COutput(I2C_SCL, I2C_CLEAR_OUT);
    SetI2CDir( I2C_SDA, I2C_INPUT);

    ReceiveData = 0; 
   for(i = 0; i < 8; i++) 
	{
		SetI2CDelay();
		SetI2COutput( I2C_SCL, I2C_SET_OUT);
		SetI2CDelayE(I2C_DELAYREAD1);
		bit_val = GetI2CSDA();
		ReceiveData |= bit_val;
		
		if(i < 7)
			ReceiveData <<= 1;
		
		SetI2CDelayE(I2C_DELAYREAD2);

		SetI2COutput( I2C_SCL, I2C_CLEAR_OUT);
		if(i==7)
		{
			SetI2COutput( I2C_SDA, I2C_CLEAR_OUT);
			SetI2CDir( I2C_SDA, I2C_OUTPUT);
			SetI2COutput( I2C_SDA, I2C_CLEAR_OUT);
		}

	}
	
	SetI2CDelay();
	return(ReceiveData);
}


static unsigned char  GetI2CACK()
{
    uint8  bit_val;
    uint8    count = 0; 

 
    SetI2COutput( I2C_SCL|I2C_SDA, I2C_CLEAR_OUT);

    SetI2CDir( I2C_SDA, I2C_INPUT);
    SetI2CDelay();  
    SetI2COutput(I2C_SCL, I2C_SET_OUT);
    SetI2CDelay();

    do{
        bit_val = GetI2CSDA();
        count++;
    }while(bit_val&(count<=5));

    SetI2CDelay();

    SetI2COutput( I2C_SCL|I2C_SDA, I2C_CLEAR_OUT);

    SetI2CDir(I2C_SDA, I2C_OUTPUT);
    SetI2CDelay();

    if(count <= 5)
        return KAL_TRUE;
    else
        return KAL_FALSE;
}



static void  SendI2CNACK()
{
   
    SetI2CDir( I2C_SDA, I2C_OUTPUT);
    SetI2COutput( I2C_SDA, I2C_SET_OUT);
    SetI2COutput( I2C_SCL, I2C_CLEAR_OUT);
    SetI2CDelay();
    SetI2COutput( I2C_SCL, I2C_SET_OUT);
    SetI2CDelay();
    SetI2COutput( I2C_SCL, I2C_CLEAR_OUT);
    SetI2COutput( I2C_SDA, I2C_CLEAR_OUT);
    SetI2CDelay();
    SetI2COutput(I2C_SCL, I2C_SET_OUT);
    SetI2CDelay();
}


static void  SendI2CACK()
{
  // set SDA to output
  SetI2CDir(I2C_SDA, I2C_OUTPUT);
  SetI2COutput((I2C_SCL | I2C_SDA), I2C_CLEAR_OUT);
  SetI2CDelay();
  SetI2COutput(I2C_SCL, I2C_SET_OUT);
  SetI2CDelay();
  SetI2COutput(I2C_SCL, I2C_CLEAR_OUT);
  SetI2CDir(I2C_SDA, I2C_INPUT);

}



static void WriteI2CSlaveAddr( uint8 address, uint8 readaccess)
{

    SetI2CLongDelay();

    StartI2C();
	
	  if(readaccess)
		  address |= 1;    
		else
		  address &= 0xFE | 0;
		
	WriteI2CData(address);  
	  
}



#define GET_I2C_ACK()   if(!GetI2CACK())\
                      {\
                        StopI2C();\
                        return KAL_FALSE;\
                      } 


/*********************************************************************
*
*from BB send data to device
**********************************************************************/
unsigned char  I2ChwWriteData(uint8 address, uint8 stopbit, uint8 *data_p, uint8 data_len)
{
    unsigned char isSuccess = KAL_TRUE;

    uint32 index = 0;

    SetI2CLongDelay();

    WriteI2CSlaveAddr( address, KAL_FALSE);
    GET_I2C_ACK();
    
    for (index = 0; index < data_len; index++)
    {
        WriteI2CData(*(data_p+index));
        GET_I2C_ACK();
    }

    if(stopbit)
        StopI2C();

    return isSuccess;
}


/*********************************************************************
*
*BB  recv data from device
**********************************************************************/
unsigned char  I2ChwReadData( uint8 address, uint8 stopbit, uint8 *data_p, uint8 data_len)
{
    unsigned char isSuccess = KAL_TRUE;
    uint8 index = 0;

    if(data_p == NULL)
        return KAL_FALSE;

    if(data_len == 0)
        return KAL_FALSE;
  
    SetI2CLongDelay();
  
    WriteI2CSlaveAddr(address, KAL_TRUE);
    GET_I2C_ACK();
    
    for(index  = 0; index < data_len-1; index++)
    {
        *(data_p+index) = ReadI2CData();
        SendI2CACK();
    }

    *(data_p+index) = ReadI2CData();
    SendI2CNACK();
    
    if(stopbit)
        StopI2C();
    
    return isSuccess;
}


/*********************************************************************
*
*init i2c
**********************************************************************/
unsigned char I2ChwInit()
{
    unsigned char  isSuccess = KAL_TRUE;

    SIMSET_I2C_CLK_OUTPUT();
    SIMSET_I2C_CLK_HIGH();
    SIMSET_I2C_DATA_OUTPUT();
    SIMSET_I2C_DATA_HIGH();

    return isSuccess;
}


/*
@func
  quectel_task_entry
@brief
  Entry function for task. 
*/
int quectel_task_entry(void)
{
    qapi_Status_t res = QAPI_OK;

	uint8	Write_slave_Address=0X36;
	
	uint8	Date_reg =0X19;

	
	unsigned char buff[2];
    unsigned char wbuf[3]={0X19,0x03,0X04};  
  
   
    qapi_Timer_Sleep(2, QAPI_TIMER_UNIT_SEC, true);
    /* Initialize debug UART and syslog */
	i2c_uart_dbg_init();  
	
    I2C_UART_DBG("\r\n===quectel i2c task entry Start!!!===\r\n");


	I2ChwInit();
	
	/*********** Read register date ********************************/

	I2ChwWriteData(Write_slave_Address,0,&Date_reg,1);
	I2ChwReadData(Write_slave_Address,1,buff,2);

	qapi_Timer_Sleep(500, QAPI_TIMER_UNIT_MSEC, true);

	I2C_UART_DBG("i2c_read: Register_Address[0x19] byte1[0x%x]  byte2[0x%x]\r\n", buff[0],buff[1]);
	
	/*********** Write register date ********************************/
	I2ChwWriteData(Write_slave_Address,1,wbuf,3);
	I2C_UART_DBG(" Write_date byte0[0x03] byte1[0x04]\r\n");
	qapi_Timer_Sleep(1000, QAPI_TIMER_UNIT_MSEC, true);

	/*********** Read register date ********************************/
	I2ChwWriteData(Write_slave_Address,0,&Date_reg,1);
	I2ChwReadData(Write_slave_Address,1,buff,2);

	qapi_Timer_Sleep(500, QAPI_TIMER_UNIT_MSEC, true);

	I2C_UART_DBG("i2c_read: Register_Address[0x19] byte1[0x%x]  byte2[0x%x]\r\n", buff[0],buff[1]);
	
	I2C_UART_DBG("\r\n===quectel i2c task entry Exit!!!===\r\n");

    return 0;
}

#endif /*__EXAMPLE_I2C__*/

