/******************************************************************************
*@file    example_stdlib.c
*@brief   example of standard lib operation
*
*  ---------------------------------------------------------------------------
*
*  Copyright (c) 2018 Quectel Technologies, Inc.
*  All Rights Reserved.
*  Confidential and Proprietary - Quectel Technologies, Inc.
*  ---------------------------------------------------------------------------
*******************************************************************************/
#if defined(__EXAMPLE_STDLIB__)
/**************************************************************************
*                                 INCLUDE
***************************************************************************/
// STANDARED LIBRARY
// C89
#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <float.h>
#include <limits.h>
#include <locale.h>
#include <math.h>
#include <setjmp.h>
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

// C95
#include <iso646.h>
#include <wchar.h>
#include <wctype.h>

// C99
//#include <complex.h>	// not supported
#include <fenv.h>
#include <inttypes.h>
#include <stdbool.h>
#include <stdint.h>
#include <tgmath.h>

// Qualcomm include
#include "qapi_uart.h"
#include "qapi_timer.h"
#include "qapi_diag.h"

// Quectel include
#include "quectel_utils.h"
#include "quectel_uart_apis.h"
#include "example_stdlib.h"

/**************************************************************************
*                                 GLOBAL
***************************************************************************/
/* uart1 rx tx buffer */
static char rx_buff[4096];	/*!!! should keep this buffer as 4K Bytes */
static char tx_buff[4096];

/* uart config para*/
static QT_UART_CONF_PARA uart_conf =
{
	NULL,
	QT_UART_PORT_02,
	tx_buff,
	sizeof(tx_buff),
	rx_buff,
	sizeof(rx_buff),
	115200
};

/* conter used to count the total run times*/
static unsigned long task_run_couter = 0;


/**************************************************************************
*                                 FUNCTION
***************************************************************************/
static void test_assert(void);
static void test_ctype(void);
static void test_errno(void);
static void test_float(void);
static void test_limits(void);
static void test_locale(void);
static void test_math(void);
static void test_setjmp(void);
static void test_signal(void);
static void test_stdarg(void);
static void test_stddef(void);
static void test_stdio(void);
static void test_stdlib(void);
static void test_string(void);
static void test_time(void);

/*
@func
  quectel_task_entry
@brief
  Entry function for task. 
*/
int quectel_task_entry(void)
{
	qapi_Timer_Sleep(3, QAPI_TIMER_UNIT_SEC, true);
	IOT_INFO("QT# quectel_task_entry");

	/// MUST SETTING
	setlocale(LC_ALL, "C");	/// <locale.h>

	/* debug port init */
	uart_init(&uart_conf);
	/* start uart receive */
	uart_recv(&uart_conf);
	/* prompt task running */
	qt_uart_dbg(uart_conf.hdlr,"\r\nSTDLIB Example");

	test_assert();	/// <assert.h>
	test_ctype();	/// <ctype.h>
	test_errno();	/// <errno.h>
	test_float();	/// <float.h>
	test_limits();	/// <limits.h>
	test_locale();	/// <locale.h>
	test_math();	/// <math.h>
	test_setjmp();	/// <setjump.h>
	test_signal();
	test_stdarg();
	test_stddef();
	test_stdio();
	test_stdlib();
	test_string();
	test_time();

	while (1)
	{
		task_run_couter ++;

		/* print log to uart */
		qt_uart_dbg(uart_conf.hdlr,"task run times : %d", task_run_couter);

		/* sleep 15 seconds */
		qapi_Timer_Sleep(10, QAPI_TIMER_UNIT_SEC, true);

	}
}

/// test <assert.h>
static void test_assert(void)
{
	int test_val = 10;

	assert(test_val > 0);
	qt_uart_dbg(uart_conf.hdlr,"test_val %d", test_val);

/// unmask and assert will take affect, module will crash.
//	assert(test_val == 0);
//	qt_uart_dbg(uart_conf.hdlr,"test_val %d", test_val);
}

/// test <ctype.h>
static void test_ctype(void)
{
 	int ret = 0;

{///isalnum
	char str01 = '1';
	char str02 = '*';
	ret = isalnum(str01);
	qt_uart_dbg(uart_conf.hdlr,"01 isalnum %d", ret);

	ret = isalnum(str02);
	qt_uart_dbg(uart_conf.hdlr,"01 isalnum %d", ret);
}	

{///isalpha
	char str01 = 'a';
	char str02 = '1';
	char str03 = 'A';

	ret = isalpha(str01);
	qt_uart_dbg(uart_conf.hdlr,"02 isalpha %d", ret);

	ret = isalpha(str02);
	qt_uart_dbg(uart_conf.hdlr,"02 isalpha %d", ret);

	ret = isalpha(str03);
	qt_uart_dbg(uart_conf.hdlr,"02 isalpha %d", ret);

}	

{///iscntrl
	char str01 = '\a';
	char str02 = '1';

	ret = iscntrl(str01);
	qt_uart_dbg(uart_conf.hdlr,"03 iscntrl %d", ret);

	ret = iscntrl(str02);
	qt_uart_dbg(uart_conf.hdlr,"03 iscntrl %d", ret);
}	

{///isdigit
	char str01 = 'a';
	char str02 = '1';

	ret = isdigit(str01);
	qt_uart_dbg(uart_conf.hdlr,"04 isdigit %d", ret);

	ret = isdigit(str02);
	qt_uart_dbg(uart_conf.hdlr,"04 isdigit %d", ret);
}	

{///isgraph
	char str01 = 'a';
	char str02 = ' ';

	ret = isgraph(str01);
	qt_uart_dbg(uart_conf.hdlr,"05 isgraph %d", ret);

	ret = isgraph(str02);
	qt_uart_dbg(uart_conf.hdlr,"05 isgraph %d", ret);
}	

{///islower
	char str01 = 'a';
	char str02 = 'A';

	ret = islower(str01);
	qt_uart_dbg(uart_conf.hdlr,"06 islower %d", ret);

	ret = islower(str02);
	qt_uart_dbg(uart_conf.hdlr,"06 islower %d", ret);
}	

{///isprint
	char str01 = 'a';
	char str02 = ' ';

	ret = isprint(str01);
	qt_uart_dbg(uart_conf.hdlr,"07 isprint %d", ret);

	ret = isprint(str02);
	qt_uart_dbg(uart_conf.hdlr,"07 isprint %d", ret);
}	

{///ispunct
	char str01 = 'a';
	char str02 = ':';

	ret = ispunct(str01);
	qt_uart_dbg(uart_conf.hdlr,"08 ispunct %d", ret);

	ret = ispunct(str02);
	qt_uart_dbg(uart_conf.hdlr,"08 ispunct %d", ret);
}	

{///isspace
	char str01 = 'a';
	char str02 = ' ';

	ret = isspace(str01);
	qt_uart_dbg(uart_conf.hdlr,"09 isspace %d", ret);

	ret = isspace(str02);
	qt_uart_dbg(uart_conf.hdlr,"09 isspace %d", ret);
}	

{///isupper
	char str01 = 'a';
	char str02 = 'A';

	ret = isupper(str01);
	qt_uart_dbg(uart_conf.hdlr,"10 isupper %d", ret);

	ret = isupper(str02);
	qt_uart_dbg(uart_conf.hdlr,"10 isupper %d", ret);
}	

{///isxdigit
	char str01 = 'a';
	char str02 = 'h';

	ret = isxdigit(str01);
	qt_uart_dbg(uart_conf.hdlr,"11 isxdigit %d", ret);

	ret = isxdigit(str02);
	qt_uart_dbg(uart_conf.hdlr,"11 isxdigit %d", ret);
}	

{///toupper
	char str01 = 'a';
	char str02 = 'A';

	qt_uart_dbg(uart_conf.hdlr,"12 toupper %d", str01);
	ret = toupper(str01);
	qt_uart_dbg(uart_conf.hdlr,"12 toupper %d", ret);

	qt_uart_dbg(uart_conf.hdlr,"13 tolower %d", str02);
	ret = tolower(str02);
	qt_uart_dbg(uart_conf.hdlr,"13 tolower %d", ret);
}
}

static void test_errno(void)
{
	/// nothing...
}

static void test_float(void)
{
	qt_uart_dbg(uart_conf.hdlr,"test_float FLT_MAX %.10e", FLT_MAX);
	qt_uart_dbg(uart_conf.hdlr,"test_float FLT_MIN %.10e", FLT_MIN);
	qt_uart_dbg(uart_conf.hdlr,"test_float FLT_MANT_DIG %.10e", FLT_MANT_DIG);
}

static void test_limits(void)
{
	qt_uart_dbg(uart_conf.hdlr,"test_limits CHAR_BIT %d", CHAR_BIT);

	qt_uart_dbg(uart_conf.hdlr,"test_limits SCHAR_MIN %d", SCHAR_MIN);
	qt_uart_dbg(uart_conf.hdlr,"test_limits SCHAR_MAX %d", SCHAR_MAX);
	qt_uart_dbg(uart_conf.hdlr,"test_limits UCHAR_MAX %d", UCHAR_MAX);

	qt_uart_dbg(uart_conf.hdlr,"test_limits SHRT_MIN %d", SHRT_MIN);
	qt_uart_dbg(uart_conf.hdlr,"test_limits SHRT_MAX %d", SHRT_MAX);

	qt_uart_dbg(uart_conf.hdlr,"test_limits INT_MIN %d", INT_MIN);
	qt_uart_dbg(uart_conf.hdlr,"test_limits INT_MAX %d", INT_MAX);

	qt_uart_dbg(uart_conf.hdlr,"test_limits CHAR_MIN %d", CHAR_MIN);
	qt_uart_dbg(uart_conf.hdlr,"test_limits CHAR_MAX %d", CHAR_MAX);

	qt_uart_dbg(uart_conf.hdlr,"test_limits LONG_MIN %ld", LONG_MIN);
	qt_uart_dbg(uart_conf.hdlr,"test_limits LONG_MAX %ld", LONG_MAX);
}

static void test_locale(void)
{
	/// nothing...
}

static void test_math(void)
{
	/// not support...
}

static void test_setjmp(void)
{
	static jmp_buf buf;
	if(!setjmp(buf))
	{
		qt_uart_dbg(uart_conf.hdlr,"test_setjmp first");
		longjmp(buf,1);
		qt_uart_dbg(uart_conf.hdlr,"test_setjmp second");
	}
	else
	{
		qt_uart_dbg(uart_conf.hdlr,"test_setjmp third");
	}
}

static void test_signal(void)
{
	/// nothing...
}

static void test_stdarg(void)
{
	/// nothing...
}

static void test_stddef(void)
{
	struct test_struct{
		char val01;
		int  val02;
		char array[20];
		long val03;
		char val04;
	};

	qt_uart_dbg(uart_conf.hdlr,"test_stddef %d", offsetof(struct test_struct, val01));
	qt_uart_dbg(uart_conf.hdlr,"test_stddef %d", offsetof(struct test_struct, val02));
	qt_uart_dbg(uart_conf.hdlr,"test_stddef %d", offsetof(struct test_struct, array));
	qt_uart_dbg(uart_conf.hdlr,"test_stddef %d", offsetof(struct test_struct, val03));
	qt_uart_dbg(uart_conf.hdlr,"test_stddef %d", offsetof(struct test_struct, val04));
}

static void test_stdio(void)
{
	char buffer[128];

	sprintf(buffer, "%s", "sprintf test");
	qt_uart_dbg(uart_conf.hdlr, "test_stdio %s", buffer);

	sscanf(buffer, "%s", "sscanf test");
	qt_uart_dbg(uart_conf.hdlr, "test_stdio %s", buffer);

}

static void test_stdlib(void)
{
	char *str_val1 = "1234";
	char *str_val2 = "123456789";
	void *pmem = NULL;

	qt_uart_dbg(uart_conf.hdlr, "test_stdlib atoi %d",  atoi(str_val1));
	qt_uart_dbg(uart_conf.hdlr, "test_stdlib atof %f",  atof(str_val1));
	qt_uart_dbg(uart_conf.hdlr, "test_stdlib atol %ld", atol(str_val2));

/// not supported.
#if 0
	pmem = malloc(1024);
	if(pmem != NULL)
		qt_uart_dbg(uart_conf.hdlr, "test_stdlib malloc %08X", pmem);
#endif
}

static void test_string(void)
{
	char *paddr = NULL;
	char *str1 = "123445654985";
	char *str2 = "565";

	paddr = strchr(str1, '5');
	if(paddr != NULL)
	{
		qt_uart_dbg(uart_conf.hdlr, "test_string strchr %d", (int)paddr-(int)str1);
	}

	paddr = strstr(str1, str2);
	if(paddr != NULL)
	{
		qt_uart_dbg(uart_conf.hdlr, "test_string strchr %d", (int)paddr-(int)str1);
	}

}

static void test_time(void)
{
	// nothing...
}


#endif /*__EXAMPLE_STDLIB__*/

