/******************************************************************************
*@file    example_task.c
*@brief   example of new task creation
*
*  ---------------------------------------------------------------------------
*
*  Copyright (c) 2018 Quectel Technologies, Inc.
*  All Rights Reserved.
*  Confidential and Proprietary - Quectel Technologies, Inc.
*  ---------------------------------------------------------------------------
*******************************************************************************/
#if defined(__EXAMPLE_TASK_CREATE__)
#include "txm_module.h"
#include "qapi_diag.h"
#include "qapi_timer.h"
#include "qapi_uart.h"
#include "quectel_utils.h"
#include "quectel_uart_apis.h"
#include "example_task.h"

/**************************************************************************
*                                 DEFINE
***************************************************************************/
#define QT_SUB1_THREAD_PRIORITY   	180
#define QT_SUB1_THREAD_STACK_SIZE 	(1024 * 16)
#define QT_SUB2_THREAD_PRIORITY   	180
#define QT_SUB2_THREAD_STACK_SIZE 	(1024 * 16)

/**************************************************************************
*                                 GLOBAL
***************************************************************************/
/* uart rx tx buffer */
static char rx_buff[1024];
static char tx_buff[1024];

/* uart config para*/
QT_UART_CONF_PARA uart1_conf =
{
	NULL,
	QT_UART_PORT_02,
	tx_buff,
	sizeof(tx_buff),
	rx_buff,
	sizeof(rx_buff),
	115200
};

/* conter used to count the total run times for main task */
static unsigned long main_thread_run_couter = 0;
static unsigned long main_thread_run_couter2 = 0;

/* thread handle */
static TX_THREAD* qt_sub1_thread_handle; 
static TX_THREAD* qt_sub2_thread_handle; 
static unsigned char qt_sub1_thread_stack[QT_SUB1_THREAD_STACK_SIZE];
static unsigned char qt_sub2_thread_stack[QT_SUB2_THREAD_STACK_SIZE];

/* TX QUEUE handle */
static TX_QUEUE tx_queue_handle;

/* TX QUEUE buffer */
static TASK_MSG task_comm[QT_Q_MAX_INFO_NUM];

/**************************************************************************
*                           FUNCTION DECLARATION
***************************************************************************/
void quectel_sub1_task_entry(ULONG para);

void quectel_sub2_task_entry(ULONG para);
/**************************************************************************
*                                 FUNCTION
***************************************************************************/
/*
@func
  uart_print
@brief
  Start a uart transmit action.
*/


/*
@func
  quectel_task_entry
@brief
  Entry function for task. 
*/
/*=========================================================================*/
int quectel_task_entry(void)
{
	int ret = -1;
	UINT status = 0;
    uint32 message_size;
	TASK_MSG qt_main_task_comm;
	qapi_Timer_Sleep(1, QAPI_TIMER_UNIT_SEC, true);

	/* uart 1 init */
	uart_init(&uart1_conf);

	/* start uart 1 receive */
	uart_recv(&uart1_conf);

	/* prompt task running */
	qt_uart_dbg(uart1_conf.hdlr,"[task_create] start task ~");

	/* create a new task */
	if(TX_SUCCESS != txm_module_object_allocate((VOID *)&qt_sub1_thread_handle, sizeof(TX_THREAD))) 
	{
		qt_uart_dbg(uart1_conf.hdlr,"[task_create] txm_module_object_allocate failed ~");
		IOT_INFO("[task_create] txm_module_object_allocate failed ~");
		return - 1;
	}

	/* create a new task : sub1 */
	ret = tx_thread_create(qt_sub1_thread_handle,
						   "Quectel Main Task Thread",
						   quectel_sub1_task_entry,
						   NULL,
						   qt_sub1_thread_stack,
						   QT_SUB1_THREAD_STACK_SIZE,
						   QT_SUB1_THREAD_PRIORITY,
						   QT_SUB1_THREAD_PRIORITY,
						   TX_NO_TIME_SLICE,
						   TX_AUTO_START
						   );
	      
	if(ret != TX_SUCCESS)
	{
		qt_uart_dbg(uart1_conf.hdlr,"[task_create] : Thread creation failed");
		IOT_INFO("[task_create] : Thread creation failed");
	}
	/* create a new task 2 */
	if(TX_SUCCESS != txm_module_object_allocate((VOID *)&qt_sub2_thread_handle, sizeof(TX_THREAD))) 
	{
		qt_uart_dbg(uart1_conf.hdlr,"[task_create] txm_module_object_allocate failed ~");
		IOT_INFO("[task_create] txm_module_object_allocate failed ~");
		return - 1;
	}
	/* create a new task : sub2 */
	ret = tx_thread_create(qt_sub2_thread_handle,
						   "Quectel Main Task Thread",
						   quectel_sub2_task_entry,
						   NULL,
						   qt_sub2_thread_stack,
						   QT_SUB2_THREAD_STACK_SIZE,
						   QT_SUB2_THREAD_PRIORITY,
						   QT_SUB2_THREAD_PRIORITY,
						   TX_NO_TIME_SLICE,
						   TX_AUTO_START
						   );
		  
	if(ret != TX_SUCCESS)
	{
		qt_uart_dbg(uart1_conf.hdlr,"[task_create] : Thread creation failed");
		IOT_INFO("[task_create] : Thread creation failed");
	}


	/* create a new queue : q_task_comm */
	status = tx_queue_create(&tx_queue_handle,
							 "q_task_comm",
							 TX_16_ULONG,
							 task_comm,
							 QT_Q_MAX_INFO_NUM *sizeof(TASK_MSG)
							 );
	if (TX_SUCCESS != status)
	{
		qt_uart_dbg(uart1_conf.hdlr, "tx_queue_create failed with status %d", status);
	}
	else
	{
		qt_uart_dbg(uart1_conf.hdlr, "tx_queue_create ok with status %d", status);
	}

	while (1)
	{

		memset(&qt_main_task_comm, 0, sizeof(qt_main_task_comm));
		/* rec data from main task by queue */
		status = tx_queue_receive(&tx_queue_handle, &qt_main_task_comm, TX_WAIT_FOREVER);
		if(TX_SUCCESS != status)
		{
			qt_uart_dbg(uart1_conf.hdlr, "[Main task_create] tx_queue_receive failed with status %d", status);
		}
		qt_uart_dbg(uart1_conf.hdlr,"[uart 1] task run times : %d,%d,%s", qt_main_task_comm.dat,qt_main_task_comm.msg_id,qt_main_task_comm.buffer);

	}

	return 0;
}

/*
@func
  quectel_sub2_task_entry
@brief
  A sub task will create by Quectel task entry. 
*/

void quectel_sub2_task_entry(ULONG para)
{
	UINT status = 0;
	TASK_MSG qt_sub2_task_comm;

	qt_uart_dbg(uart1_conf.hdlr,"[task_create] start sub2 task ~");

	while (1)
	{
		main_thread_run_couter2 ++; 
		
		qt_sub2_task_comm.msg_id=2;
		qt_sub2_task_comm.dat=main_thread_run_couter2;
		strcpy(qt_sub2_task_comm.name,"Task2");
		strcpy(qt_sub2_task_comm.buffer,"Task2 send Successful");
		/* send data to sub1 task by queue */
		status = tx_queue_send(&tx_queue_handle, &qt_sub2_task_comm,TX_WAIT_FOREVER);
		if (TX_SUCCESS != status)
		{
			qt_uart_dbg(uart1_conf.hdlr, "[task_create 2] tx_queue_send failed with status %d", status);
		} 
	}
}

/*
@func
  quectel_sub1_task_entry
@brief
  A sub task will create by Quectel task entry. 
*/
void quectel_sub1_task_entry(ULONG para)
{
	UINT status = 0;
	TASK_MSG qt_sub1_task_comm;

	qt_uart_dbg(uart1_conf.hdlr,"[task_create] start sub1 task ~");

	while (1)
	{
		main_thread_run_couter ++; 
		
		qt_sub1_task_comm.msg_id=1;
		qt_sub1_task_comm.dat=main_thread_run_couter;
		strcpy(qt_sub1_task_comm.name,"Task1");
		strcpy(qt_sub1_task_comm.buffer,"Task1 send Successful");
	
		/* send data to sub1 task by queue */
		status = tx_queue_send(&tx_queue_handle, &qt_sub1_task_comm,TX_WAIT_FOREVER );
		if (TX_SUCCESS != status)
		{
			qt_uart_dbg(uart1_conf.hdlr, "[task_create 1 ] tx_queue_send failed with status %d", status);
		}
	}
}

#endif /*__EXAMPLE_TASK_CREATE__*/


