/******************************************************************************
*@file    example_atc_pipe.c
*@brief   example of atc pipe operation
*
*  ---------------------------------------------------------------------------
*
*  Copyright (c) 2018 Quectel Technologies, Inc.
*  All Rights Reserved.
*  Confidential and Proprietary - Quectel Technologies, Inc.
*  ---------------------------------------------------------------------------
*******************************************************************************/
#if defined(__EXAMPLE_UART_ATC__)
#include "txm_module.h"
#include "qapi_diag.h"
#include "qapi_timer.h"
#include "qapi_uart.h"
#include "qapi_quectel.h"
#include "quectel_utils.h"
#include "quectel_uart_apis.h"
#include "example_uart_atc.h"

/**************************************************************************
*                                 DEFINE
***************************************************************************/


/**************************************************************************
*                                 GLOBAL
***************************************************************************/
/* uart1 rx tx buffer */
static char rx1_buff[1024];
static char tx1_buff[1024];

/* uart config para*/
static QT_UART_CONF_PARA uart1_conf =
{
	NULL,
	QT_UART_PORT_02,
	tx1_buff,
	sizeof(tx1_buff),
	rx1_buff,
	sizeof(rx1_buff),
	115200
};

qapi_at_stream_id_t stream_id_0, stream_id_1;
static char at_cmd_rsp[3096] = {0};
TX_EVENT_FLAGS_GROUP atc_pipe_sig_handle;

static TX_QUEUE tx_queue_handle;
static TASK_MSG task_comm[QT_Q_MAX_INFO_NUM];



/*===========================================================================

                           Static & global Variable Declarations

===========================================================================*/

static void uart_rx_cb_at(uint32_t num_bytes, void *cb_data)
{
	QT_UART_CONF_PARA *uart_conf = (QT_UART_CONF_PARA*)cb_data;
	TASK_MSG rxcb;

	qapi_Status_t status;
		
	if(num_bytes == 0)
	{
		uart_recv(uart_conf);
		return;
	}
	else if(num_bytes >= uart_conf->rx_len)
	{
		num_bytes = uart_conf->rx_len;
	}
	
#ifdef QT_UART_ECHO_ENABLE
	uart_print(uart_conf, uart_conf->rx_buff, num_bytes);
#endif
	IOT_DEBUG("QT# uart_rx_cb data [%d][%s]", num_bytes, uart_conf->rx_buff);
//	memset(uart_conf->rx_buff, 0, uart_conf->rx_len);
	uart_recv(uart_conf);

	status = tx_queue_send(&tx_queue_handle, &rxcb,TX_WAIT_FOREVER );
	if (TX_SUCCESS != status)
	{
		qt_uart_dbg(uart1_conf.hdlr, "[task_create 1 ] tx_queue_send failed with status %d", status);
	}
}



void uart_init_at(QT_UART_CONF_PARA *uart_conf)
{
	qapi_Status_t status;
	qapi_UART_Open_Config_t uart_cfg;
	QAPI_Flow_Control_Type uart_fc_type = QAPI_FCTL_OFF_E;
	
	uart_cfg.baud_Rate			= uart_conf->baudrate;
	uart_cfg.enable_Flow_Ctrl	= QAPI_FCTL_OFF_E;
	uart_cfg.bits_Per_Char		= QAPI_UART_8_BITS_PER_CHAR_E;
	uart_cfg.enable_Loopback	= 0;
	uart_cfg.num_Stop_Bits		= QAPI_UART_1_0_STOP_BITS_E;
	uart_cfg.parity_Mode		= QAPI_UART_NO_PARITY_E;
	uart_cfg.rx_CB_ISR			= (qapi_UART_Callback_Fn_t)&uart_rx_cb_at;
	uart_cfg.tx_CB_ISR			= NULL;
	
	status = qapi_UART_Open(&uart_conf->hdlr, uart_conf->port_id, &uart_cfg);
	IOT_DEBUG("QT# qapi_UART_Open [%d] status %d", uart_conf->port_id, status);
	
	status = qapi_UART_Power_On(uart_conf->hdlr);
	IOT_DEBUG("QT# qapi_UART_Power_On [%d] status %d", uart_conf->port_id, status);
	
	status = qapi_UART_Ioctl(uart_conf->hdlr, QAPI_SET_FLOW_CTRL_E, &uart_fc_type);
	IOT_DEBUG("QT# qapi_UART_Ioctl [%d] status %d", uart_conf->port_id, status);	
}


void uart_recv_at(QT_UART_CONF_PARA *uart_conf)
{
	qapi_Status_t status;
	status = qapi_UART_Receive(uart_conf->hdlr, uart_conf->rx_buff, uart_conf->rx_len, (void*)uart_conf);
	IOT_DEBUG("QT# qapi_UART_Receive [%d] status %d", (qapi_UART_Port_Id_e)uart_conf->port_id, status);
}


static int ReceiveUart()
{
	int i = 0;
			
    if(rx1_buff[0] == '\0')
	{
		return 0;
	}
	
	for(i = 0;i < strlen(rx1_buff);i++)
	{
	
		if((rx1_buff[i] != ' ')&&(rx1_buff[i] != '\n')&&(rx1_buff[i] != '\r'))
		{
			break;
		}
	}
	
     if((strlen(rx1_buff) - i)>= 2)
	 {
		 if ((( rx1_buff[i] == 'A')||( rx1_buff[i] == 'a'))&&(( rx1_buff[i+1] == 'T')||( rx1_buff[i+1] == 't')||( rx1_buff[i+1] == '/')))
		 {
		 	if((strlen(rx1_buff) >=2 )&&(rx1_buff[strlen(rx1_buff) - 2] == '\r')&&(rx1_buff[strlen(rx1_buff) - 1] == '\n'))
		 	{
				return 2;
		 	}
		 }	 
	}

 	return 1;
}




static void callback_atc_pipe_0(qapi_at_pipe_data_t *data)
{
	ULONG sig_event = 0;


	tx_event_flags_get(&atc_pipe_sig_handle, SIG_EVG_ATPIPE_SENDING_E, TX_OR, &sig_event, TX_NO_WAIT);
	if(sig_event & SIG_EVG_ATPIPE_SENDING_E)
	{
		tx_event_flags_set(&atc_pipe_sig_handle, ~SIG_EVG_ATPIPE_SENDING_E, TX_AND);
		memset(at_cmd_rsp, 0, sizeof(at_cmd_rsp));
	}
	
	strcat(at_cmd_rsp, data->data);


	if((strstr(data->data,"\r\nOK") != NULL)||(strstr(data->data,"\r\nERROR") != NULL))
	{
		tx_event_flags_set(&atc_pipe_sig_handle, SIG_EVG_ATPIPE_REC_OK_E, TX_OR);
	}
}


int quectel_task_entry
(
    void
)
{
	qapi_Status_t status = QAPI_OK;
	ULONG sig_event = 0;
	TASK_MSG rxdata;

	qapi_Status_t ATCmdstate = 0;

	qapi_Timer_Sleep(3, QAPI_TIMER_UNIT_SEC, true);

	/* Create event signal handle and clear signals */
	tx_event_flags_create(&atc_pipe_sig_handle, "atc_pipe_sig_handle");
	tx_event_flags_set(&atc_pipe_sig_handle, 0, TX_AND);

	/* uart 1 init */
	uart_init_at(&uart1_conf);
	/* start uart 1 receive */
	
	uart_recv_at(&uart1_conf);
	
	/* prompt task running */
	qt_uart_dbg(uart1_conf.hdlr,"[f] start task ~");

	/* Open first ATC port */
 	status = qapi_QT_Apps_AT_Port_Open(QAPI_AT_PORT_0, &stream_id_0, callback_atc_pipe_0);
	if(status != QAPI_QT_ERR_OK)
	{
		qt_uart_dbg(uart1_conf.hdlr,"[atc pipe 0] status[0x%x]", status);
	}
	

	
	tx_event_flags_set(&atc_pipe_sig_handle, SIG_EVG_ATPIPE_SENDING_E, TX_OR);
	status = qapi_QT_Apps_Send_AT(stream_id_0, "ATI\r\n");
	if(status != QAPI_QT_ERR_OK)
	{
		qt_uart_dbg(uart1_conf.hdlr,"[atc pipe 1] status[0x%x]", status);
	}

	tx_event_flags_get(&atc_pipe_sig_handle, SIG_EVG_ATPIPE_REC_OK_E, TX_OR, &sig_event, TX_WAIT_FOREVER);
	if(sig_event & SIG_EVG_ATPIPE_REC_OK_E)
	{
		qt_uart_dbg(uart1_conf.hdlr,"[atc pipe 1] rsp[%s]", at_cmd_rsp);
		tx_event_flags_set(&atc_pipe_sig_handle, ~SIG_EVG_ATPIPE_REC_OK_E, TX_AND);
	}


	/* create a new queue : q_task_comm */
	status = tx_queue_create(&tx_queue_handle,
							 "q_task_comm",
							 TX_16_ULONG,
							 task_comm,
							 QT_Q_MAX_INFO_NUM *sizeof(TASK_MSG)
							 );
	
	if (TX_SUCCESS != status)
	{
		qt_uart_dbg(uart1_conf.hdlr, "tx_queue_create failed with status %d", status);
	}
	else
	{
		qt_uart_dbg(uart1_conf.hdlr, "tx_queue_create ok with status %d", status);
	}

	

	while(1)
	{
	
	     status = tx_queue_receive(&tx_queue_handle, &rxdata, TX_WAIT_FOREVER);

		 if(TX_SUCCESS != status)
		 {
			qt_uart_dbg(uart1_conf.hdlr, "[Main task_create] tx_queue_receive failed with status %d", status);
		 }

		 ATCmdstate = ReceiveUart();

		 if(ATCmdstate == 0)
		 {
		 	 qapi_Timer_Sleep(1, QAPI_TIMER_UNIT_SEC, true);
		 }
		 else if(ATCmdstate == 2)
		 {
	
				tx_event_flags_set(&atc_pipe_sig_handle, SIG_EVG_ATPIPE_SENDING_E, TX_OR);
				status = qapi_QT_Apps_Send_AT(stream_id_0, rx1_buff);
				if(status != QAPI_QT_ERR_OK)
				{
					qt_uart_dbg(uart1_conf.hdlr,"[atc pipe] status[0x%x]", status);
				}
				tx_event_flags_get(&atc_pipe_sig_handle, SIG_EVG_ATPIPE_REC_OK_E, TX_OR, &sig_event, TX_WAIT_FOREVER);
				if(sig_event & SIG_EVG_ATPIPE_REC_OK_E)
				{
					qt_uart_dbg(uart1_conf.hdlr,"[atc pipe] rsp[%s]", at_cmd_rsp);
					tx_event_flags_set(&atc_pipe_sig_handle, ~SIG_EVG_ATPIPE_REC_OK_E, TX_AND);
					memset(rx1_buff, 0, sizeof(rx1_buff));
				}

				qapi_Timer_Sleep(1, QAPI_TIMER_UNIT_SEC, true);
		 }
		 else
		 {
		 	qt_uart_dbg(uart1_conf.hdlr,"rx1_buff = %s", rx1_buff);
			memset(rx1_buff, 0, sizeof(rx1_buff));
			qapi_Timer_Sleep(1, QAPI_TIMER_UNIT_SEC, true);
		 }
	}
}

#endif /*__EXAMPLE_PSM__*/

